use xcb;
use ui::font;
use ui::color;

use std::rc::Rc;
use std::sync::Arc;

pub struct DrawContext {
    conn: Arc<xcb::Connection>,
    picture: xcb::render::Picture,
    pen: xcb::render::Picture,
    fonts: Rc<font::FontLoader>,
    bg_color: xcb::render::Color
}

impl DrawContext {
    pub fn new(conn: Arc<xcb::Connection>, picture: xcb::render::Picture, fonts: Rc<font::FontLoader>) -> DrawContext {
        let pen = conn.generate_id();
        xcb::render::create_solid_fill(
            &conn,
            pen,
            color::WHITE
        );

        DrawContext {
            conn: conn,
            picture: picture,
            pen: pen,
            fonts: fonts,
            bg_color: color::BLACK
        }
    }

    pub fn draw_bg(&self, x: u16, width: u16) {
        xcb::render::fill_rectangles(
            &self.conn,
            xcb::render::PICT_OP_SRC as u8,
            self.picture,
            self.bg_color,
            &[xcb::Rectangle::new(x as i16, 0, width, 20)]
        );
    }

    pub fn measure_text(&self, name: &str) -> u16 {
        let text = self.fonts.create_renderable_text(name);
        text.width
    }

    pub fn set_bg_color(&mut self, color: xcb::render::Color) {
        self.bg_color = color;
    }


    pub fn draw_text(&self, name: &str, x: u16) {
        if !name.is_empty() {
            let text = self.fonts.create_renderable_text(name);
            let baseline = 20 - self.fonts.default_offset(20);
            text.render(&self.conn, self.pen, self.picture, x, baseline as u16);
        }
    }

    pub fn draw_text_with_clipping(&self, name: &str, x: u16, width: u16) {
        xcb::render::set_picture_clip_rectangles(
            &self.conn,
            self.picture,
            0, 0,
            &[xcb::Rectangle::new(x as i16, 0, width, 20)]
        );

        self.draw_text(name, x);

        xcb::render::change_picture(
            &self.conn,
            self.picture,
            &[(xcb::render::CP_CLIP_MASK, xcb::render::PICTURE_NONE)]
        );
    }

    pub fn flush(&self) {
        self.conn.flush();
    }
}
