mod title;
mod tray;
mod sensors;
mod wm;
mod spacer;
mod music;

use xcb;
use mpd;
use std::sync::mpsc;

pub use self::title::title;
pub use self::tray::tray;
pub use self::sensors::sensors;
pub use self::spacer::spacer;
pub use self::music::mpd;
pub use self::wm::bspwm;

pub type MessageSender = mpsc::Sender<Message>;

pub enum Message {
    Relayout,
    Update,
    Quit,
    MousePress(u16),
    BspwmEvent(String),
    MpdEvent(mpd::status::State, Option<mpd::song::Song>),
    XcbEvent(xcb::GenericEvent)
}

pub trait Widget {
    fn init(&mut self) {}
    fn render(&mut self, x: u16, width: u16);
    fn width(&mut self) -> u16;
    fn fit_width(&self) -> bool {
        false
    }
    fn handle_event(&mut self, _event: &Message) -> bool {
        false
    }
    fn finish(&mut self) {}
}
